@@ -43,6 +43,8 @@ char *		rb_find_user_data_file	(const char *name);
 char *		rb_find_user_cache_file	(const char *name);
 char *		rb_find_plugin_data_file (GObject *plugin,
 					 const char *name);
+char *		rb_find_plugin_resource (GObject *plugin,
+					 const char *name);
 
 char *		rb_canonicalise_uri	(const char *uri);
 
@@ -58,6 +60,7 @@ gboolean	rb_uri_is_writable	(const char *uri);
 gboolean	rb_uri_is_local		(const char *uri);
 gboolean	rb_uri_is_hidden	(const char *uri);
 gboolean	rb_uri_could_be_podcast (const char *uri, gboolean *is_opml);
+gboolean	rb_uri_is_descendant	(const char *uri, const char *ancestor);
 char *		rb_uri_make_hidden      (const char *uri);
 char *		rb_uri_get_dir_name	(const char *uri);
 char *		rb_uri_get_short_path_name (const char *uri);
@@ -90,7 +93,7 @@ GFile *		rb_file_find_extant_parent (GFile *file);
 
 gboolean	rb_uri_create_parent_dirs (const char *uri, GError **error);
 
-void		rb_file_helpers_init	(gboolean uninstalled);
+void		rb_file_helpers_init	(void);
 void		rb_file_helpers_shutdown(void);
 
 char *		rb_uri_get_filesystem_type (const char *uri, char **mount_point);
