@@ -25,8 +25,6 @@
  *
  */
 
-#define __EXTENSIONS__
-
 #include "config.h"
 
 #include <string.h>
@@ -214,8 +212,8 @@ music_dirs_done (RBAndroidSource *source)
 static void
 enum_files_cb (GObject *obj, GAsyncResult *result, gpointer data)
 {
-	RBAndroidSource *source = RB_ANDROID_SOURCE (data);
-	RBAndroidSourcePrivate *priv = GET_PRIVATE(source);
+	RBAndroidSource *source;
+	RBAndroidSourcePrivate *priv;
 	GFileEnumerator *e = G_FILE_ENUMERATOR (obj);
 	GError *error = NULL;
 	GFileInfo *info;
@@ -225,10 +223,16 @@ enum_files_cb (GObject *obj, GAsyncResult *result, gpointer data)
 	files = g_file_enumerator_next_files_finish (e, result, &error);
 	if (error != NULL) {
 		rb_debug ("error listing files: %s", error->message);
-		music_dirs_done (source);
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			music_dirs_done (RB_ANDROID_SOURCE (data));
+		}
+		g_clear_error (&error);
 		return;
 	}
 
+	source = RB_ANDROID_SOURCE (data);
+	priv = GET_PRIVATE(source);
+
 	if (files == NULL) {
 		priv->scanned++;
 		g_object_unref (e);
@@ -276,19 +280,23 @@ enum_files_cb (GObject *obj, GAsyncResult *result, gpointer data)
 static void
 enum_child_cb (GObject *obj, GAsyncResult *result, gpointer data)
 {
-	RBAndroidSource *source = RB_ANDROID_SOURCE (data);
-	RBAndroidSourcePrivate *priv = GET_PRIVATE(source);
+	RBAndroidSource *source;
+	RBAndroidSourcePrivate *priv;
 	GFileEnumerator *e;
 	GError *error = NULL;
 
 	e = g_file_enumerate_children_finish (G_FILE (obj), result, &error);
 	if (e == NULL) {
 		rb_debug ("enum error: %s", error->message);
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			music_dirs_done (RB_ANDROID_SOURCE (data));
+		}
 		g_clear_error (&error);
-		music_dirs_done (source);
 		return;
 	}
 
+	source = RB_ANDROID_SOURCE (data);
+	priv = GET_PRIVATE(source);
 	g_file_enumerator_next_files_async (e, 64, G_PRIORITY_DEFAULT, priv->cancel, enum_files_cb, source);
 }
 
@@ -370,6 +378,10 @@ actually_load (RBAndroidSource *source)
 	char *name;
 	char *label;
 
+	if (priv->loaded) {
+		rb_debug ("already loading");
+		return;
+	}
 	priv->loaded = TRUE;
 	rb_media_player_source_load (RB_MEDIA_PLAYER_SOURCE (source));
 
@@ -721,7 +733,7 @@ build_device_uri (RBAndroidSource *source, RhythmDBEntry *entry, const char *med
 		if (p != NULL) {
 			*p = '\0';
 		}
-		file = g_strdup_printf ("%s%s", title, ext);
+		file = g_strdup_printf (G_DIR_SEPARATOR_S "%s%s", title, ext);
 	}
 
 	if (file == NULL) {
@@ -764,110 +776,73 @@ build_device_uri (RBAndroidSource *source, RhythmDBEntry *entry, const char *med
 	return uri;
 }
 
-static char *
-impl_build_dest_uri (RBTransferTarget *target,
-		     RhythmDBEntry *entry,
-		     const char *media_type,
-		     const char *extension)
-{
-	return g_strdup (RB_ENCODER_DEST_TEMPFILE);
-}
-
 static void
-track_copy_cb (GObject *src, GAsyncResult *res, gpointer data)
+impl_track_upload (RBTransferTarget *target,
+		   RhythmDBEntry *entry,
+		   const char *dest,
+		   guint64 filesize,
+		   const char *media_type,
+		   GError **error)
 {
-	RBAndroidSource *source = RB_ANDROID_SOURCE (data);
-	RBAndroidSourcePrivate *priv = GET_PRIVATE (source);
-	RhythmDBEntryType *entry_type;
-	RBShell *shell;
-	RhythmDB *db;
-	GFile *dest;
-	char *uri;
-	GError *error = NULL;
+	RBAndroidSource *source = RB_ANDROID_SOURCE (target);
+	char *realdest;
+	GFile *dfile, *sfile;
 
-	if (g_task_propagate_boolean (G_TASK (res), &error)) {
+	realdest = build_device_uri (source, entry, media_type, rb_gst_media_type_to_extension (media_type));
+	dfile = g_file_new_for_uri (realdest);
+	sfile = g_file_new_for_uri (dest);
 
-		dest = G_FILE (src);
-		uri = g_file_get_uri (dest);
-
-		g_object_get (source, "shell", &shell, NULL);
-		g_object_get (shell, "db", &db, NULL);
-		g_object_unref (shell);
-
-		g_object_get (source, "entry-type", &entry_type, NULL);
-		rhythmdb_add_uri_with_types (db,
-					     uri,
-					     entry_type,
-					     priv->ignore_type,
-					     priv->error_type);
-		g_object_unref (entry_type);
-		g_object_unref (db);
-		g_free (uri);
-
-		update_free_space (source);
-	} else {
-		rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
-	}
-
-	g_clear_error (&error);
-	g_object_unref (src);
-	g_object_unref (source);
-}
-
-static void
-copy_track_task (GTask *task, gpointer pdest, gpointer psource, GCancellable *cancel)
-{
-	GFile *source = G_FILE (psource);
-	GFile *dest = G_FILE (pdest);
-	GError *error = NULL;
-	char *uri;
-
-	uri = g_file_get_uri (dest);
-	rb_debug ("creating parent dirs for %s", uri);
-	if (rb_uri_create_parent_dirs (uri, &error) == FALSE) {
-		g_file_delete (source, NULL, NULL);
-		g_free (uri);
-		g_task_return_error (task, error);
+	rb_debug ("creating parent dirs for %s", realdest);
+	if (rb_uri_create_parent_dirs (realdest, error) == FALSE) {
+		g_file_delete (sfile, NULL, NULL);
+		g_free (realdest);
+		g_object_unref (dfile);
+		g_object_unref (sfile);
 		return;
 	}
-	rb_debug ("moving %s", uri);
-	g_free (uri);
 
-	g_file_move (source,
-		     dest,
-		     G_FILE_COPY_OVERWRITE,
-		     NULL,
-		     NULL,
-		     NULL,
-		     &error);
-	if (error) {
-		g_file_delete (source, NULL, NULL);
-		g_task_return_error (task, error);
-	} else {
-		g_task_return_boolean (task, TRUE);
+	rb_debug ("moving %s to %s", dest, realdest);
+	if (g_file_move (sfile, dfile, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, error) == FALSE) {
+		g_file_delete (sfile, NULL, NULL);
 	}
+
+	g_free (realdest);
+	g_object_unref (dfile);
+	g_object_unref (sfile);
 }
 
 static gboolean
 impl_track_added (RBTransferTarget *target,
 		  RhythmDBEntry *entry,
 		  const char *dest,
-		  guint64 filesize,
+		  guint64 dest_size,
 		  const char *media_type)
 {
 	RBAndroidSource *source = RB_ANDROID_SOURCE (target);
+	RBAndroidSourcePrivate *priv = GET_PRIVATE (source);
+	RhythmDBEntryType *entry_type;
+	RBShell *shell;
+	RhythmDB *db;
 	char *realdest;
-	GFile *dfile, *sfile;
-	GTask *task;
+
+	g_object_get (source, "shell", &shell, NULL);
+	g_object_get (shell, "db", &db, NULL);
+
+	g_object_get (source, "entry-type", &entry_type, NULL);
 
 	realdest = build_device_uri (source, entry, media_type, rb_gst_media_type_to_extension (media_type));
-	dfile = g_file_new_for_uri (realdest);
-	sfile = g_file_new_for_uri (dest);
+	rhythmdb_add_uri_with_types (db,
+				     realdest,
+				     entry_type,
+				     priv->ignore_type,
+				     priv->error_type);
 	g_free (realdest);
 
-	task = g_task_new (dfile, NULL, track_copy_cb, g_object_ref (source));
-	g_task_set_task_data (task, sfile, g_object_unref);
-	g_task_run_in_thread (task, copy_track_task);
+	update_free_space (source);
+
+	g_object_unref (entry_type);
+	g_object_unref (shell);
+	g_object_unref (db);
 	return FALSE;
 }
 
@@ -1031,6 +1006,11 @@ impl_dispose (GObject *object)
 {
 	RBAndroidSourcePrivate *priv = GET_PRIVATE (object);
 
+	if (priv->cancel != NULL) {
+		g_cancellable_cancel (priv->cancel);
+		g_clear_object (&priv->cancel);
+	}
+
 	if (priv->db != NULL) {
 		if (priv->ignore_type != NULL) {
 			rhythmdb_entry_delete_by_type (priv->db, priv->ignore_type);
@@ -1080,7 +1060,7 @@ rb_android_device_source_init (RBDeviceSourceInterface *interface)
 static void
 rb_android_transfer_target_init (RBTransferTargetInterface *interface)
 {
-	interface->build_dest_uri = impl_build_dest_uri;
+	interface->track_upload = impl_track_upload;
 	interface->track_added = impl_track_added;
 }
 
