@@ -26,9 +26,7 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 #include <glib.h>
@@ -94,9 +92,10 @@ struct _RBDaapPlugin
 
 	GSimpleAction *new_share_action;
 
-	DMAPMdnsBrowser *mdns_browser;
+	DmapMdnsBrowser *mdns_browser;
 
-	DACPShare *dacp_share;
+	DmapControlShare *dacp_share;
+	gboolean dacp_share_started;
 
 	GHashTable *source_lookup;
 
@@ -146,7 +145,7 @@ RB_DEFINE_PLUGIN(RB_TYPE_DAAP_PLUGIN,
 static void
 rb_daap_plugin_init (RBDaapPlugin *plugin)
 {
-	GSettings *daap_settings;
+	g_autoptr(GSettings) daap_settings = NULL;
 
 	rb_debug ("RBDaapPlugin initialising");
 	rb_daap_src_set_plugin (G_OBJECT (plugin));
@@ -155,7 +154,6 @@ rb_daap_plugin_init (RBDaapPlugin *plugin)
 
 	daap_settings = g_settings_new ("org.gnome.rhythmbox.plugins.daap");
 	plugin->dacp_settings = g_settings_get_child (daap_settings, "dacp");
-	g_object_unref (daap_settings);
 
 	rb_register_gst_plugin ();
 }
@@ -165,7 +163,7 @@ impl_activate (PeasActivatable *bplugin)
 {
 	RBDaapPlugin *plugin = RB_DAAP_PLUGIN (bplugin);
 	gboolean no_registration;
-	RBShell *shell;
+	g_autoptr(RBShell) shell = NULL;
 	GApplication *app;
 
 	plugin->shutdown = FALSE;
@@ -206,20 +204,21 @@ impl_activate (PeasActivatable *bplugin)
 		rb_daap_sharing_init (shell);
 
 	plugin->dacp_share = rb_daap_create_dacp_share (G_OBJECT (plugin));
+	plugin->dacp_share_started = FALSE;
 	if (g_settings_get_boolean (plugin->dacp_settings, "enable-remote")) {
-		dacp_share_start_lookup (plugin->dacp_share);
+		GError *error = NULL;
+		dmap_control_share_start_lookup_compat (plugin->dacp_share, &error);
+		plugin->dacp_share_started = TRUE;
 	}
 
 	register_daap_dbus_iface (plugin);
-
-	g_object_unref (shell);
 }
 
 static void
 impl_deactivate	(PeasActivatable *bplugin)
 {
 	RBDaapPlugin *plugin = RB_DAAP_PLUGIN (bplugin);
-	RBShell *shell;
+	g_autoptr(RBShell) shell = NULL;
 
 	rb_debug ("Shutting down DAAP plugin");
 
@@ -238,29 +237,11 @@ impl_deactivate	(PeasActivatable *bplugin)
 		stop_browsing (plugin);
 	}
 
-	if (plugin->settings) {
-		g_object_unref (plugin->settings);
-		plugin->settings = NULL;
-	}
-
-	g_object_unref (plugin->dacp_share);
-
-	if (plugin->preferences) {
-		gtk_widget_destroy (plugin->preferences);
-		plugin->preferences = NULL;
-	}
-
-	if (plugin->builder) {
-		g_object_unref (plugin->builder);
-		plugin->builder = NULL;
-	}
-
-	if (plugin->bus) {
-		g_object_unref (plugin->bus);
-		plugin->bus = NULL;
-	}
-
-	g_object_unref (shell);
+	g_clear_object (&plugin->settings);
+	g_clear_object (&plugin->dacp_share);
+	g_clear_pointer (&plugin->preferences, gtk_widget_destroy);
+	g_clear_object (&plugin->builder);
+	g_clear_object (&plugin->bus);
 }
 
 /* DAAP share icons */
@@ -291,50 +272,65 @@ find_source_by_service_name (RBDaapPlugin *plugin,
 }
 
 static void
-mdns_service_added (DMAPMdnsBrowser *browser,
-		    DMAPMdnsBrowserService *service,
+mdns_service_added (DmapMdnsBrowser *browser,
+		    DmapMdnsService *service,
 		    RBDaapPlugin *plugin)
 {
 	RBSource *source;
-	RBShell *shell;
+	g_autoptr(RBShell) shell = NULL;
+	gchar *service_name = NULL;
+	gchar *name         = NULL;
+	gchar *host         = NULL;
+	guint port;
+	gboolean password_protected;
+
+	rb_dmap_mdns_service_get_compat(service,
+	                                &service_name,
+	                                &name,
+	                                &host,
+	                                &port,
+	                                &password_protected);
+
 
 	rb_debug ("New service: %s name=%s host=%s port=%u password=%d",
-		   service->service_name,
-		   service->name,
-		   service->host,
-		   service->port,
-		   service->password_protected);
+		   service_name,
+		   name,
+		   host,
+		   port,
+		   password_protected);
 
-	source = find_source_by_service_name (plugin, service->service_name);
+	source = find_source_by_service_name (plugin, service_name);
 
 	if (source == NULL) {
 		g_object_get (plugin, "object", &shell, NULL);
 
 		source = rb_daap_source_new (shell,
 					     G_OBJECT (plugin),
-					     service->service_name,
-					     service->name,
-					     service->host,
-					     service->port,
-					     service->password_protected);
-		g_hash_table_insert (plugin->source_lookup, g_strdup (service->service_name), source);
+					     service_name,
+					     name,
+					     host,
+					     port,
+					     password_protected);
+		g_hash_table_insert (plugin->source_lookup, g_strdup (service_name), source);
 		rb_shell_append_display_page (shell,
 					      RB_DISPLAY_PAGE (source),
 					      RB_DISPLAY_PAGE_GROUP_SHARED);
-
-		g_object_unref (shell);
 	} else {
 		g_object_set (source,
-			      "name", service->name,
-			      "host", service->host,
-			      "port", service->port,
-			      "password-protected", service->password_protected,
+			      "name", name,
+			      "host", host,
+			      "port", port,
+			      "password-protected", password_protected,
 			      NULL);
 	}
+
+	g_free (service_name);
+	g_free (name);
+	g_free (host);
 }
 
 static void
-mdns_service_removed (DMAPMdnsBrowser *browser,
+mdns_service_removed (DmapMdnsBrowser *browser,
 		      const char        *service_name,
 		      RBDaapPlugin	*plugin)
 {
@@ -370,7 +366,7 @@ start_browsing (RBDaapPlugin *plugin)
 		return;
 	}
 
-	plugin->mdns_browser = dmap_mdns_browser_new (DMAP_MDNS_BROWSER_SERVICE_TYPE_DAAP);
+	plugin->mdns_browser = dmap_mdns_browser_new (DMAP_MDNS_SERVICE_TYPE_DAAP);
 	if (plugin->mdns_browser == NULL) {
 		g_warning ("Unable to start mDNS browsing");
 		return;
@@ -403,7 +399,7 @@ start_browsing (RBDaapPlugin *plugin)
 static void
 stop_browsing (RBDaapPlugin *plugin)
 {
-	GError *error;
+	g_autoptr(GError) error = NULL;
 
 	if (plugin->mdns_browser == NULL) {
 		return;
@@ -419,23 +415,25 @@ stop_browsing (RBDaapPlugin *plugin)
 
 	error = NULL;
 	dmap_mdns_browser_stop (plugin->mdns_browser, &error);
-	if (error != NULL) {
+	if (error != NULL)
 		g_warning ("Unable to stop mDNS browsing: %s", error->message);
-		g_error_free (error);
-	}
 
-	g_object_unref (plugin->mdns_browser);
-	plugin->mdns_browser = NULL;
+	g_clear_object (&plugin->mdns_browser);
 }
 
 static void
 dacp_settings_changed_cb (GSettings *settings, const char *key, RBDaapPlugin *plugin)
 {
 	if (g_strcmp0 (key, "enable-remote") == 0) {
-		if (g_settings_get_boolean (settings, key)) {
-			dacp_share_start_lookup (plugin->dacp_share);
-		} else {
-			dacp_share_stop_lookup (plugin->dacp_share);
+		GError *error = NULL;
+		if (g_settings_get_boolean (settings, key) != plugin->dacp_share_started) {
+			if (plugin->dacp_share_started) {
+				dmap_control_share_stop_lookup_compat (plugin->dacp_share, &error);
+				plugin->dacp_share_started = FALSE;
+			} else {
+				dmap_control_share_start_lookup_compat (plugin->dacp_share, &error);
+				plugin->dacp_share_started = TRUE;
+			}
 		}
 	}
 }
@@ -473,7 +471,7 @@ new_daap_share_location_added_cb (RBURIDialog *dialog,
 	char *host;
 	char *p;
 	int port = 3689;
-	DMAPMdnsBrowserService service;
+	DmapMdnsService *service;
 
 	host = g_strdup (location);
 	p = strrchr (host, ':');
@@ -483,17 +481,18 @@ new_daap_share_location_added_cb (RBURIDialog *dialog,
 	}
 
 	rb_debug ("adding manually specified DAAP share at %s", location);
-	service.name = (char *) location;
-	service.host = (char *) host;
-	service.service_name = service.name;
-	service.port = port;
-	service.password_protected = FALSE;
+	service = rb_dmap_mdns_service_new_compat(location,
+	                                          location,
+	                                          host,
+	                                          port,
+	                                          FALSE);
+
 	mdns_service_added (NULL,
-			    &service,
+			    service,
 			    plugin);
 
 	g_free (host);
-
+	rb_dmap_mdns_service_free_compat(service);
 }
 
 static void
@@ -589,15 +588,12 @@ static void
 forget_remotes_button_toggled_cb (GtkToggleButton *button,
 				  gpointer data)
 {
-	GSettings *dacp_settings;
-	GSettings *daap_settings;
+	g_autoptr(GSettings) dacp_settings = NULL;
+	g_autoptr(GSettings) daap_settings = NULL;
 
 	daap_settings = g_settings_new ("org.gnome.rhythmbox.plugins.daap");
 	dacp_settings = g_settings_get_child (daap_settings, "dacp");
 	g_settings_reset (dacp_settings, "known-remotes");
-
-	g_object_unref (dacp_settings);
-	g_object_unref (daap_settings);
 }
 
 static gboolean
@@ -605,10 +601,10 @@ share_name_entry_focus_out_event_cb (GtkEntry *entry,
 				     GdkEventFocus *event,
 				     gpointer data)
 {
-	GSettings  *settings;
+	g_autoptr(GSettings) settings = NULL;
+	g_autofree char *old_name = NULL;
 	gboolean    changed;
 	const char *name;
-	char       *old_name;
 
 	settings = g_settings_new ("org.gnome.rhythmbox.sharing");
 	name = gtk_entry_get_text (entry);
@@ -628,9 +624,6 @@ share_name_entry_focus_out_event_cb (GtkEntry *entry,
 		g_settings_set_string (settings, "share-name", name);
 	}
 
-	g_free (old_name);
-	g_object_unref (settings);
-
 	return FALSE;
 }
 
@@ -639,10 +632,10 @@ share_password_entry_focus_out_event_cb (GtkEntry *entry,
 					 GdkEventFocus *event,
 					 RBDaapPlugin *plugin)
 {
-	GSettings  *settings;
+	g_autoptr(GSettings) settings = NULL;
+	g_autofree char *old_pw = NULL;
 	gboolean    changed;
 	const char *pw;
-	char       *old_pw;
 
 	pw = gtk_entry_get_text (entry);
 	settings = g_settings_new ("org.gnome.rhythmbox.sharing");
@@ -662,9 +655,6 @@ share_password_entry_focus_out_event_cb (GtkEntry *entry,
 		g_settings_set_string (settings, "share-password", pw);
 	}
 
-	g_free (old_pw);
-	g_object_unref (settings);
-
 	return FALSE;
 }
 
@@ -778,16 +768,27 @@ daap_dbus_method_call (GDBusConnection *connection,
 	}
 
 	if (g_strcmp0 (method_name, "AddDAAPSource") == 0) {
-		DMAPMdnsBrowserService service = {0,};
-		g_variant_get (parameters, "(&s&su)", &service.name, &service.host, &service.port);
-		service.password_protected = FALSE;
-		service.service_name = service.name;
+		DmapMdnsService *service;
+		gchar *name = NULL;
+		gchar *host = NULL;
+		guint port;
 
-		rb_debug ("adding DAAP source %s (%s:%d)", service.name, service.host, service.port);
-		mdns_service_added (NULL, &service, plugin);
+		g_variant_get (parameters, "(&s&su)", &name, &host, &port);
+
+		service = rb_dmap_mdns_service_new_compat(name,
+							  name,
+							  host,
+							  port,
+							  FALSE);
+
+		rb_debug ("adding DAAP source %s (%s:%d)", name, host, port);
+		mdns_service_added (NULL, service, plugin);
 
 		g_dbus_method_invocation_return_value (invocation, NULL);
 
+		g_free(name);
+		g_free(host);
+		rb_dmap_mdns_service_free_compat(service);
 	} else if (g_strcmp0 (method_name, "RemoveDAAPSource") == 0) {
 		const char *service_name;
 
@@ -808,8 +809,8 @@ static const GDBusInterfaceVTable daap_dbus_vtable = {
 static void
 register_daap_dbus_iface (RBDaapPlugin *plugin)
 {
-	GError *error = NULL;
-	GDBusNodeInfo *node_info;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GDBusNodeInfo) node_info = NULL;
 	GDBusInterfaceInfo *iface_info;
 
 	if (plugin->dbus_intf_id != 0) {
@@ -821,7 +822,6 @@ register_daap_dbus_iface (RBDaapPlugin *plugin)
 		plugin->bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
 		if (plugin->bus == NULL) {
 			rb_debug ("Unable to register DAAP DBus interface: %s", error->message);
-			g_clear_error (&error);
 			return;
 		}
 	}
@@ -829,7 +829,6 @@ register_daap_dbus_iface (RBDaapPlugin *plugin)
 	node_info = g_dbus_node_info_new_for_xml (rb_daap_dbus_iface, &error);
 	if (error != NULL) {
 		rb_debug ("Unable to parse DAAP DBus spec: %s", error->message);
-		g_clear_error (&error);
 		return;
 	}
 
@@ -842,12 +841,8 @@ register_daap_dbus_iface (RBDaapPlugin *plugin)
 						   g_object_ref (plugin),
 						   g_object_unref,
 						   &error);
-	if (error != NULL) {
+	if (error != NULL)
 		rb_debug ("Unable to register DAAP DBus interface: %s", error->message);
-		g_clear_error (&error);
-	}
-
-	g_dbus_node_info_unref (node_info);
 }
 
 static void
