@@ -43,11 +43,11 @@ struct RBRhythmDBQueryModelDMAPDbAdapterPrivate {
 
 typedef struct ForeachAdapterData {
 	gpointer data;
-	GHFunc func;
+	DmapIdRecordFunc func;
 } ForeachAdapterData;
 
-static DMAPRecord *
-rb_rhythmdb_query_model_dmap_db_adapter_lookup_by_id (const DMAPDb *db,
+static DmapRecord *
+rb_rhythmdb_query_model_dmap_db_adapter_lookup_by_id (const DmapDb *db,
 						      guint id)
 {
 	g_error ("Not implemented");
@@ -61,7 +61,7 @@ foreach_adapter (GtkTreeModel *model,
 		 gpointer data)
 {
 	gulong id;
-	DMAPRecord *record;
+	DmapRecord *record;
 	RhythmDBEntry *entry;
 	ForeachAdapterData *foreach_adapter_data;
 
@@ -71,9 +71,15 @@ foreach_adapter (GtkTreeModel *model,
 	foreach_adapter_data = data;
 	record = DMAP_RECORD (rb_daap_record_new (entry));
 
-	foreach_adapter_data->func (GUINT_TO_POINTER (id),
+#ifdef LIBDMAPSHARING_COMPAT
+	foreach_adapter_data->func (GUINT_TO_POINTER(id),
 				    record,
 				    foreach_adapter_data->data);
+#else
+	foreach_adapter_data->func (id,
+				    record,
+				    foreach_adapter_data->data);
+#endif
 
 	g_object_unref (record);
 	rhythmdb_entry_unref (entry);
@@ -82,9 +88,9 @@ foreach_adapter (GtkTreeModel *model,
 }
 
 static void
-rb_rhythmdb_query_model_dmap_db_adapter_foreach	(const DMAPDb *db,
-					 GHFunc func,
-				         gpointer data)
+rb_rhythmdb_query_model_dmap_db_adapter_foreach	(const DmapDb *db,
+                                                 DmapIdRecordFunc func,
+                                                 gpointer data)
 {
 	ForeachAdapterData *foreach_adapter_data;
 
@@ -102,15 +108,15 @@ rb_rhythmdb_query_model_dmap_db_adapter_foreach	(const DMAPDb *db,
 }
 
 static gint64
-rb_rhythmdb_query_model_dmap_db_adapter_count (const DMAPDb *db)
+rb_rhythmdb_query_model_dmap_db_adapter_count (const DmapDb *db)
 {
 	g_assert (RB_RHYTHMDB_QUERY_MODEL_DMAP_DB_ADAPTER (db)->priv->model != NULL); 
 	return gtk_tree_model_iter_n_children (
 		GTK_TREE_MODEL (RB_RHYTHMDB_QUERY_MODEL_DMAP_DB_ADAPTER (db)->priv->model), NULL);
 }
 
-static guint
-rb_rhythmdb_query_model_dmap_db_adapter_add (DMAPDb *db, DMAPRecord *record)
+guint
+rb_rhythmdb_query_model_dmap_db_adapter_add (DmapDb *db, DmapRecord *record, GError **error)
 {
 	g_error ("Not implemented");
 	return 0;
@@ -136,11 +142,11 @@ rb_rhythmdb_query_model_dmap_db_adapter_class_finalize (RBRhythmDBQueryModelDMAP
 static void
 rb_rhythmdb_query_model_dmap_db_adapter_interface_init (gpointer iface, gpointer data)
 {
-	DMAPDbIface *dmap_db = iface;
+	DmapDbInterface *dmap_db = iface;
 
 	g_assert (G_TYPE_FROM_INTERFACE (dmap_db) == DMAP_TYPE_DB);
 
-	dmap_db->add = rb_rhythmdb_query_model_dmap_db_adapter_add;
+	dmap_db->add = rb_rhythmdb_query_model_dmap_db_adapter_add_compat;
 	dmap_db->lookup_by_id = rb_rhythmdb_query_model_dmap_db_adapter_lookup_by_id;
 	dmap_db->foreach = rb_rhythmdb_query_model_dmap_db_adapter_foreach;
 	dmap_db->count = rb_rhythmdb_query_model_dmap_db_adapter_count;
